unit AssocArray;

{ Tablica asocjacyjna by bidok (michu-bidok@o2.pl)
  jesli chcesz wykorzystac ten unit w celach komercyjnych
  najpier skontaktuj sie ze mna }

interface

type
  PAssocItem = ^TAssocItem;
  TAssocItem = record
    Next: PAssocItem;
    Name: Variant;
    Value: Pointer;
  end;

  TAssocArray = class
  protected
    function GetItem(const Name: Variant): Pointer;
    procedure SetItem(const Name: Variant; Value: Pointer);
  private
    AList: PAssocItem;
    ALast: PAssocItem;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Clear;
    procedure AddItem(Name: string; Value: Pointer);
    procedure RemItem(Name: string);

    function GetNextItem(var Item: PAssocItem): Boolean;

    property Items[const Name: Variant]: Pointer read GetItem write SetItem; default;
  end;

implementation

constructor TAssocArray.Create;
begin
  inherited Create;

  AList := nil;
  ALast := nil;
end;

destructor TAssocArray.Destroy;
begin
  Clear;

  inherited Destroy;
end;

function TAssocArray.GetNextItem(var Item: PAssocItem): Boolean;
begin
  Result := False;
  if AList = nil then Exit;

  if Item = nil then
  begin
    Item := AList;
    Result := True;
    Exit;
  end;

  if (Item.Next <> nil) then
  begin
    Item := Item^.Next;
    Result := True;
  end;
end;

procedure TAssocArray.Clear;
var
  Temp, Temp2: PAssocItem;
begin
  if AList = nil then Exit;

  Temp := AList;
  while Temp <> nil do
  begin
    Temp2 := Temp^.Next;
    Dispose(Temp);
    Temp := Temp2;
  end;
  AList := nil;
  ALast := nil;
end;

function TAssocArray.GetItem;
var
  Temp: PAssocItem;
begin
  Result := nil;
  if AList = nil then Exit;

  Temp := AList;
  while (Temp <> nil) and (Temp^.Name <> Name) do
    Temp := Temp.Next;

  if Temp = nil then Exit;
  Result := Temp.Value;
end;

procedure TAssocArray.SetItem;
var
  Temp: PAssocItem;
begin
  Temp := AList;
  while (Temp <> nil) and (Temp.Name <> Name) do
    Temp := Temp.Next;

  if Temp = nil then
     AddItem(Name, Value) else
  begin
    if Value = nil then
       RemItem(Name) else
       Temp^.Value := Value;
  end;
end;

procedure TAssocArray.AddItem;
var
  Temp: PAssocItem;
begin
  if Value = nil then Exit;

  New(Temp);
  Temp^.Next := nil;
  Temp^.Name := Name;
  Temp^.Value := Value;

  if (AList = nil) then
  begin
    AList := Temp;
    ALast := Temp;
  end else
  begin
    ALast^.Next := Temp;
    ALast := Temp;
  end;
end;

procedure TAssocArray.RemItem;
var
  Temp, Temp2: PAssocItem;
begin
  if AList = nil then Exit;

  Temp2 := nil;
  Temp := AList;
  while (Temp <> nil) and (Temp.Name <> Name) do
  begin
    Temp2 := Temp;
    Temp := Temp.Next;
  end;

  if Temp = nil then Exit;

  if Temp2 <> nil then
     Temp2^.Next := Temp^.Next else
     AList := Temp^.Next;
  Dispose(Temp);
end;

end.
